package pps.core.prediction.service;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.prediction.entity.PlantPredictedPowerDataEnt;
import pps.core.prediction.entity.PlantPredictedPowerLongTermDataEnt;
import pps.core.prediction.mapper.PlantPredictedPowerDataMapper;
import pps.core.prediction.mapper.PlantPredictedPowerLongTermDataMapper;
import xstartup.base.XContext;
import xstartup.data.XListResult;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 统计分析基类
 *
 * @author ZWT
 * @date 2023/09/28 10:21
 */
public class StatisticAnalysisBaseService {

    /**
     * 获得光伏发电
     *
     * @param context     上下文
     * @param dateType    日期类型
     * @param plantIdList 工厂id列表
     * @return {@link List}
     */
    public List getPhotovoltaicPowerGeneration(XContext context, Integer dateType, List<String> plantIdList) {
        List powerGenerationList = null;
        //取当前时间
        DateTime date = DateUtil.date();
        PlantPredictedPowerDataMapper powerDataMapper;
        switch (dateType) {
            case 0:
                powerDataMapper = context.getBean(PlantPredictedPowerDataMapper.class);
                powerGenerationList = powerDataMapper.selectList(new QueryWrapper<PlantPredictedPowerDataEnt>()
                        .select("hour_time",
                                "SUM( power ) AS power")
                        .lambda()
                        .eq(PlantPredictedPowerDataEnt::getYearTime, String.valueOf(date.year()))
                        .eq(PlantPredictedPowerDataEnt::getMonthTime, BaseUtils.getMonthString(date.month() + 1))
                        .eq(PlantPredictedPowerDataEnt::getDayTime, String.valueOf(date.dayOfMonth()))
                        .in(PlantPredictedPowerDataEnt::getPlantId, plantIdList)
                        .groupBy(PlantPredictedPowerDataEnt::getHourTime)
                        .orderByAsc(PlantPredictedPowerDataEnt::getHourTime)
                );
                break;
            case 1:
                powerDataMapper = context.getBean(PlantPredictedPowerDataMapper.class);
                powerGenerationList = powerDataMapper.selectList(new QueryWrapper<PlantPredictedPowerDataEnt>()
                        .select("hour_time",
                                "SUM( power ) AS power")
                        .lambda()
                        .in(PlantPredictedPowerDataEnt::getPlantId, plantIdList)
                        .between(PlantPredictedPowerDataEnt::getDataDate,
                                date.toString(BusinessConstant.DATE_FORMAT_DAY),
                                DateUtil.offsetDay(date, 7).toString(BusinessConstant.DATE_FORMAT_DAY))
                        .groupBy(PlantPredictedPowerDataEnt::getHourTime)
                        .orderByAsc(PlantPredictedPowerDataEnt::getHourTime)
                );
                break;
            case 2:
                PlantPredictedPowerLongTermDataMapper longTermDataMapper = context.getBean(PlantPredictedPowerLongTermDataMapper.class);
                powerGenerationList = longTermDataMapper.selectList(new QueryWrapper<PlantPredictedPowerLongTermDataEnt>()
                        .select("hour_time",
                                "SUM( power ) AS power")
                        .lambda()
                        .in(PlantPredictedPowerLongTermDataEnt::getPlantId, plantIdList)
                        .between(PlantPredictedPowerLongTermDataEnt::getDataDate,
                                date.toString(BusinessConstant.DATE_FORMAT_DAY),
                                DateUtil.offsetDay(date, 30).toString(BusinessConstant.DATE_FORMAT_DAY))
                        .groupBy(PlantPredictedPowerLongTermDataEnt::getHourTime)
                        .orderByAsc(PlantPredictedPowerLongTermDataEnt::getHourTime)
                );
                break;
            default:
        }
        return powerGenerationList;
    }

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context 上下文
     * @param ouId    ou id
     * @return {@link List}<{@link String}>
     */
    public List<String> getOrgIdsByPath(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(ouId);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult().stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
    }

    /**
     * 获取光伏发电厂
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link GetBasePhotovoltaicPlantCloudOutput}>
     */
    public List<GetBasePhotovoltaicPlantCloudOutput> getPhotovoltaicPlantByParam(XContext context, GetBasePhotovoltaicPlantCloudInput input) {
        BasePhotovoltaicPlantCloudService cloudService = context.getBean(BasePhotovoltaicPlantCloudService.class);
        XListResult<GetBasePhotovoltaicPlantCloudOutput> result = cloudService.getBasePhotovoltaicPlantList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

}
